import 'package:flutter/material.dart';

/// Diferent Types Of Animation.
enum AnimationType {
  normal,
  fadeIn,
  rotation,
  scaleRotation,
  scale,
  size,
  slideRight,
  slideLeft,
  slideTop,
  slideBottom
}

/// Main Class, [context] It is the BuildContext of  tha Application at that Moment.
/// [childPage] It is the Widget or Screen to Navigate.
/// [animate] It is the Type of Animation to use.
/// [duration] It is the Time You Want the Animation To Last.
/// [replacement] If this Property is True, the Previous Screen is Replaced by the New Screen.
class AnimateNavigationTrasitionDo {
  final BuildContext context;
  final Widget childPage;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  AnimateNavigationTrasitionDo(
      {required this.context,
      required this.childPage,
      this.animation = AnimationType.normal,
      this.duration = const Duration(milliseconds: 300),
      this.replacement = false}) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        _fadeInTransition();
        break;
      case AnimationType.slideRight:
        _slideRightTransition();
        break;
      case AnimationType.rotation:
        _rotationTransition();
        break;
      case AnimationType.scaleRotation:
        _scaleRotateTransition();
        break;
      case AnimationType.scale:
        _scaleTransition();
        break;
      case AnimationType.size:
        _sizeTransition();
        break;
      case AnimationType.slideLeft:
        _slideLeftTransition();
        break;
      case AnimationType.slideTop:
        _slideTopTraansition();
        break;
      case AnimationType.slideBottom:
        _slideBottomTransition();
        break;
    }
  }

  /// Normal Screen Push.
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Replacement Screen Push.
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.childPage);

    (this.replacement) ? _pushReplacementPage(route) : _pushPage(route);
  }

  ///#region Code for Transitions

  void _fadeInTransition() {
    final transition = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.childPage,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
            child: child,
            opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
                CurvedAnimation(parent: animation, curve: Curves.easeOut)),
          );
        });

    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }

  void _rotationTransition() {
    final transition = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.childPage,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) => RotationTransition(
              turns: Tween<double>(begin: 0.0, end: 1.0).animate(
                  CurvedAnimation(parent: animation, curve: Curves.linear)),
              child: child,
            ));

    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }

  void _scaleRotateTransition() {
    final transition = PageRouteBuilder(
      pageBuilder: (_, __, ___) => this.childPage,
      transitionDuration: this.duration,
      transitionsBuilder: (_, animation, __, child) => ScaleTransition(
        scale: Tween<double>(begin: 0.0, end: 1.0).animate(
            CurvedAnimation(parent: animation, curve: Curves.fastOutSlowIn)),
        child: RotationTransition(
          turns: Tween<double>(begin: 0.0, end: 1.0).animate(
              CurvedAnimation(parent: animation, curve: Curves.linear)),
          child: child,
        ),
      ),
    );
    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }

  void _scaleTransition() {
    final transition = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.childPage,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) => ScaleTransition(
            scale: Tween<double>(begin: 0.0, end: 1.0).animate(CurvedAnimation(
                parent: animation, curve: Curves.fastOutSlowIn)),
            child: child));

    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }

  void _sizeTransition() {
    final transition = PageRouteBuilder(
        pageBuilder: (_, __, ___) => childPage,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) => Align(
              child: SizeTransition(sizeFactor: animation, child: child),
            ));

    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }

  void _slideRightTransition() {
    final transition = PageRouteBuilder(
      pageBuilder: (_, __, ___) => this.childPage,
      transitionDuration: this.duration,
      transitionsBuilder: (_, animation, __, child) => SlideTransition(
        position: Tween<Offset>(
          begin: const Offset(-1, 0),
          end: Offset.zero,
        ).animate(animation),
        child: child,
      ),
    );

    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }

  void _slideLeftTransition() {
    final transition = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.childPage,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) => SlideTransition(
              position:
                  Tween<Offset>(begin: const Offset(1, 0), end: Offset.zero)
                      .animate(animation),
              child: child,
            ));
    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }

  void _slideTopTraansition() {
    final transition = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.childPage,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) => SlideTransition(
            position: Tween<Offset>(begin: const Offset(0, 1), end: Offset.zero)
                .animate(animation)));
    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }

  void _slideBottomTransition() {
    final transition = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.childPage,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) => SlideTransition(
            position:
                Tween<Offset>(begin: const Offset(0, -1), end: Offset.zero)
                    .animate(animation),
            child: child));

    (this.replacement)
        ? _pushReplacementPage(transition)
        : _pushPage(transition);
  }

  ///#endregion
}
